package items

import (
	"context"
	"testing"

	"git.perx.ru/perxis/perxis-go/pkg/environments"
	"git.perx.ru/perxis/perxis-go/pkg/errors"
	"git.perx.ru/perxis/perxis-go/pkg/options"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestBatchProcessor(t *testing.T) {
	t.Run("EmptyResults", func(t *testing.T) {
		itemssvc := &Stub{
			FindResult: func(req StubFindRequest) StubFindResult {
				return StubFindResult{Items: nil, Total: 0, Error: nil}
			},
		}

		b := &BatchProcessor{
			Items:        itemssvc,
			SpaceID:      "sp",
			EnvID:        environments.DefaultEnvironment,
			CollectionID: "col",
			FindOptions: &FindOptions{
				Regular:     true,
				Hidden:      true,
				Templates:   true,
				FindOptions: *options.NewFindOptions(0, 10),
			},
			Filter: NewFilter("a > 5"),
		}

		var counter int
		_, err := b.Do(context.Background(), func(batch []*Item) error { counter++; return nil })
		require.NoError(t, err)
		assert.Equal(t, 0, counter)
	})

	t.Run("With FindOptions", func(t *testing.T) {
		itemssvc := &Stub{
			FindResult: func(req StubFindRequest) StubFindResult {
				fo := MergeFindOptions(req.Options...)
				return StubFindResult{Items: make([]*Item, fo.Limit), Total: 1000, Error: nil}
			},
		}
		b := &BatchProcessor{
			Items:        itemssvc,
			SpaceID:      "sp",
			EnvID:        environments.DefaultEnvironment,
			CollectionID: "col",
			FindOptions:  &FindOptions{FindOptions: *options.New(0, 25)},
		}

		var counter int
		_, err := b.Do(context.Background(), func(batch []*Item) error { counter++; return nil })
		require.NoError(t, err)
		assert.Equal(t, 1000/25, counter)
	})

	t.Run("With FindOptions2", func(t *testing.T) {

		f := &FindOptions{
			Deleted:         true,
			Hidden:          true,
			Regular:         true,
			Templates:       true,
			TranslationsIDs: []string{"*"},
			LocaleID:        "en",
		}

		itemssvc := &Stub{
			FindResult: func(req StubFindRequest) StubFindResult {
				fo := req.Options[0]
				if !assert.Equal(t, f, fo) {
					return StubFindResult{Items: nil, Total: 0, Error: errors.New("FindOptions not equal")}
				}
				return StubFindResult{Items: make([]*Item, fo.Limit), Total: 25, Error: nil}
			},
		}

		f.FindOptions = *options.New(0, 25)
		b := &BatchProcessor{
			Items:        itemssvc,
			SpaceID:      "sp",
			EnvID:        environments.DefaultEnvironment,
			CollectionID: "col",
			FindOptions:  f,
		}

		var counter int
		_, err := b.Do(context.Background(), func(batch []*Item) error { counter++; return nil })
		require.NoError(t, err)
	})
}
